use crate::{config::ServerConfig, server::Server};
use std::sync::Arc;

pub struct ServerBuilder {
    pub config: ServerConfig,
}

impl ServerBuilder {
    pub fn address(mut self, ip: &str, port: u16) -> Self {
        self.config.ip = ip.to_string();
        self.config.port = port;
        self
    }

    pub fn enable_private_ips(self) -> Self {
        self.set_private_ips(true)
    }

    pub fn set_private_ips(mut self, enabled: bool) -> Self {
        self.config.private_ips = enabled;
        self.update_ip_filter_state();

        self
    }

    pub fn ips(mut self, ips: Vec<String>) -> Self {
        self.config.ips = ips;
        self.update_ip_filter_state();

        self
    }

    pub fn api_key(mut self, api_key: &str) -> Self {
        self.config.api_key = Some(api_key.to_string());
        self
    }

    pub fn build(self) -> Server {
        Server {
            config: Arc::new(self.config),
        }
    }

    fn update_ip_filter_state(&mut self) {
        self.config.ip_filter = self.config.private_ips || !self.config.ips.is_empty();
    }
}
